from typing import Dict

from langchain_core.utils import get_from_dict_or_env
from pydantic.v1 import root_validator, BaseModel
from tmdbv3api import TMDb, Movie
import requests


class TMDBWrapper(BaseModel):
    openai_api_key: str
    tmdb_api_key: str

    class Config:
        extra = "forbid"

    # noinspection PyMethodParameters
    @root_validator(pre=True)
    def validate_environment(cls, values: Dict) -> Dict:
        """Validate that api key and endpoint exists in environment."""
        tmdb_api_key = get_from_dict_or_env(
            values, "tmdb_api_key", "TMDB_API_KEY"
        )
        values["tmdb_api_key"] = tmdb_api_key

        openai_api_key = get_from_dict_or_env(
            values, "openai_api_key", "OPENAI_API_KEY"
        )
        values["openai_api_key"] = openai_api_key

        return values

    # region(ISO 3166-1)
    def get_movie_now_playing(self, max_results: int = 5, max_snippet_length: int = 100, region=None):
        tmdb = TMDb()
        tmdb.api_key = self.tmdb_api_key
        tmdb.language = 'en'

        movie = Movie()
        new_movies = movie.now_playing(region)
        limited_results = []
        for result in new_movies:
            limited_result = {
                "snippet": result,
            }
            limited_results.append(limited_result)
        return limited_results

    # region(ISO 3166-1)
    def get_movie_popular(self, region=None):
        tmdb = TMDb()
        tmdb.api_key = self.tmdb_api_key
        tmdb.language = 'en'

        movie = Movie()
        new_movies = movie.popular(region)
        return new_movies



wrapper = TMDBWrapper(tmdb_api_key='86c9f91ac5004fa70f21db5a34b50de5', openai_api_key='empty')
now_playing = wrapper.get_movie_now_playing()
print(now_playing)



"""Tool for the Movie API (popular, now playing)."""

from typing import Optional, Type

from langchain_core.callbacks import CallbackManagerForToolRun
from langchain_core.pydantic_v1 import BaseModel, Field
from langchain_core.tools import BaseTool

from wktools_langflow.wrapper.tmdb_wrapper import TMDBWrapper


class MovieQueryInput(BaseModel):
    """Input for the MovieQuery tool."""

    # query: str = Field(description="query to look up on movie")
    # region(ISO 3166-1)
    region: str = Field(description="region to look up on movie in ISO 3166-1 format", default="us")


class MovieQueryRun(BaseTool):
    """Tool that search movie, now playing, popular."""

    name: str = "movie"
    description: str = (
        "A wrapper for movie. "
        "Useful for when you need to answer questions about current movies. "
        "Now playing, popular, or search by movie name. "
    )
    api_wrapper: TMDBWrapper

    args_schema: Type[BaseModel] = MovieQueryInput

    def _run(
        self,
        region: str,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> str:
        """Use the Wikipedia tool."""
        input_data = self.args_schema(region=region)  # Create an instance of MovieQueryInput
        return self.api_wrapper.get_movie_popular(region=input_data)





from typing import Dict

from langchain_core.utils import get_from_dict_or_env
from pydantic.v1 import root_validator, BaseModel
import requests
import json


class LoanWrapper(BaseModel):
    openai_api_key: str
    loan_account_number: str = None
    user_id: str = None

    class Config:
        extra = "forbid"

    # noinspection PyMethodParameters
    @root_validator(pre=True)
    def validate_environment(cls, values: Dict) -> Dict:
        """Validate that api key and endpoint exists in environment."""
        # user_id = get_from_dict_or_env(
        #     values, "user_id", "USER_ID"
        # )
        # values["user_id"] = user_id
        #
        # loan_account_number = get_from_dict_or_env(
        #     values, "loan_account_number", "LOAN_ACCOUNT_NUMBER"
        # )
        # values["loan_account_number"] = loan_account_number


        openai_api_key = get_from_dict_or_env(
            values, "openai_api_key", "OPENAI_API_KEY"
        )
        values["openai_api_key"] = openai_api_key

        return values


    def get_user_loan_profiles(self):
        url = f"http://localhost:8000/api/loans/user/{self.user_id}"
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        try:
            response = requests.get(url, headers=headers)
            response.raise_for_status()  # Raise an exception for HTTP errors
            print(response.json())

            limited_results = []
            for result in json.loads(response.content):
                limited_result = {
                "snippet": result,
                }
                limited_results.append(limited_result)

            return limited_results
        except requests.exceptions.RequestException as e:
            print(f"An error occurred: {e}")

    # get_user_loan_profile_by_loan_account_number
    def get_user_loan_profile_by_loan_account_number(self, loan_account_number):
        url = f"http://localhost:8000/api/loans/loan-account/{loan_account_number}"
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }

        try:
            response = requests.get(url, headers=headers)
            response.raise_for_status()  # Raise an exception for HTTP errors
            print(response.json())

            limited_results = []
            for result in json.loads(response.content):
                limited_result = {
                "snippet": result,
                }
                limited_results.append(limited_result)

            return limited_results
        except requests.exceptions.RequestException as e:
            print(f"An error occurred: {e}")

        def update_outstanding(self):
            url = f"http://localhost:8000/api/loans/update-outstanding"
            headers = {
                "Accept": "application/json",
                "Content-Type": "application/json"
            }
            data = json.dumps(self.update_outstanding_request)

            try:
                response = requests.post(url, headers=headers, data=data)
                response.raise_for_status()  # Raise an exception for HTTP errors
                print(response.json())
                return response.json()
            except requests.exceptions.RequestException as e:
                print(f"An error occurred: {e}")


"""Tool for the Loan API (get user's loan profiles)."""

from typing import Optional, Type

from langchain_core.callbacks import CallbackManagerForToolRun
from langchain_core.pydantic_v1 import BaseModel, Field
from langchain_core.tools import BaseTool

from wktools_langflow.wrapper.loan_wrapper import LoanWrapper


class LoanQueryInput(BaseModel):
    """Input for the LoanQueryInput tool."""

    user_id: str = Field(description="user's id", default="wk")
    loan_account_number: str = Field(description="user's loan account number", default="111222333")

class UpdateOutstandingRequest(BaseModel):
    loan_account_number: str
    pay_amount: float

# class LoanQueryRun(BaseTool):
#     """Tool for the Loan API (get user's loan profiles)."""
#
#     name: str = "loan"
#     description: str = (
#         "A wrapper for loan. "
#         "Useful for when you need to answer questions about to get  current user's all loan profiles "
#         "Get user's all loan profiles.")
#     api_wrapper: LoanWrapper
#
#     args_schema: Type[BaseModel] = LoanQueryInput
#
#     def _run(
#         self,
#         run_manager: Optional[CallbackManagerForToolRun] = None,
#     ) -> str:
#         """Use the Loan tool."""
#         return self.api_wrapper.get_user_loan_profiles()

class LoanQueryRun(BaseTool):
    """Tool for the Loan API (get user's loan profiles)."""
    name: str = "get_user_loan_profile_list"
    description: str = (
        "A wrapper for getting loan profile list. "
        "Useful for when you need to answer questions about to get  current user's all loan profiles "
        "Get user's all loan profiles.")
    api_wrapper: LoanWrapper

    args_schema: Type[BaseModel] = LoanQueryInput

    def _run(
        self,
        run_manager: Optional[CallbackManagerForToolRun] = None,
        user_id: str = None,
    ) -> str:
        """Use the Loan tool."""
        # self.user_id = self.api_wrapper.user_id
        return self.api_wrapper.get_user_loan_profiles()

    # get_user_loan_profile_by_loan_account_number
class GetUserLoanProfileByLoanAccountNumber(BaseTool):
    """Tool for the Loan API (get user's loan profiles by loan account number)."""

    name: str = "get_user_loan_profile_by_loan_account_number"
    description: str = (
        "A wrapper for getting loan profile by providing loan account number. "
        "Useful for when you need to answer questions about to get current user's loans by loan account number."
        "Get user's loan profiles by loan account number.")
    api_wrapper: LoanWrapper

    args_schema: Type[BaseModel] = LoanQueryInput

    def _run(
        self,
        run_manager: Optional[CallbackManagerForToolRun] = None,
        loan_account_number: str = None,
    ) -> str:
        """Use the Loan tool."""
        # self.loan_account_number = self.api_wrapper.loan_account_number
        return self.api_wrapper.get_user_loan_profile_by_loan_account_number()

#     update_outstanding
class UpdateLoanOutStandingBalancdeQueryRun(BaseTool):
    """Tool for the Loan API (update user's loan outstanding amount)."""

    name: str = "loan"
    description: str = (
        "A wrapper for loan. "
        "Useful for when you need to answer questions about current user's loans. "
        "Update user's loan outstanding amount by loan account when user pays.")
    api_wrapper: LoanWrapper

    args_schema: Type[BaseModel] = UpdateOutstandingRequest

    def _run(
        self,
        run_manager: Optional[CallbackManagerForToolRun] = None,
    ) -> str:
        """Use the Loan tool."""
        return self.api_wrapper.get_user_loan_profiles(UpdateOutstandingRequest(loan_account_number=self.args_schema.loan_account_number, pay_amount=self.args_schema.pay_amount))
